	SUBROUTINE HUBPOT
**********************************************************
*  Computes Hubbard correction to the LDA potential in   *
*  the LDA+U scheme.                                     *
*  The important parameter is ISCHM=0,11,12,13,...       *
*  in MODULE COM_HUBDAT. See formulae below for a        *
*  particular scheme.                                    *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL,ONLY: IDMF                              !!! from readini
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB   !!! from readini
      USE COM_PANPAR ,ONLY: LTPAN,LSPAN                       !!! from readini
      USE COM_HUBDAT ,ONLY: NHUB,MCRL,LCRL,ISCHM              !!! from readhub
      USE COM_HUBCOM ,ONLY: DHUB,VHUB,DLDA,VCNS               !!! from readhub
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: UMAT(:,:)
	REAL*8, ALLOCATABLE :: GMAT(:,:)
	REAL*8, ALLOCATABLE :: UBAR(:,:)
	REAL*8, ALLOCATABLE :: GBAR(:,:)
	COMPLEX*16, ALLOCATABLE :: DBAR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DAVR(:,:)
	DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	IF(IDMF.NE.1)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(UMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(GMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(UBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(GBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(DBAR(0:LMAXBMEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DAVR(LIMB2MEM,NATOMMEM))
C DO GLOBALLY OVER ATOMS
	DO 1000 IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
	CALL HUBMAT(IATOM,UMAT,GMAT,UBAR,GBAR) ! set Hubbard and Exchange matrices
C  SET AVERAGE OCCUPANCIES <Nup> and <Ndn>
	DO IORBS=1,NORBS                          !!! over iorbs
	DO IUPDN=1,NSPIN                          !!! over spins
	DO L=0,ILSB(ISORT)                        !!! over l
	DBAR(L,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	DO M=-L,L                                 !!! over m
	LM=L*(L+1)+M+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	DBAR(L,IATOM,IUPDN,IORBS)=DBAR(L,IATOM,IUPDN,IORBS)+
     &DHUB(IU,IATOM,IUPDN,IORBS)
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
C  SET AVERAGE OVER SPINS ONLY
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DAVR(IU,IATOM)=(DHUB(IU,IATOM,1,1)+
     &DHUB(IU,IATOM,NSPIN,1))/2.D0
	ENDDO                                     !!! over m
	ENDDO                                     !!! over k
	ENDDO                                     !!! over l
C  SET HUBBARD CORRECTION TO THE POTENTIAL
	DO IUPDN=1,NSPIN                          !!! over spins
	IF(IUPDN.EQ.1)IUPDN1=NSPIN
	IF(IUPDN.EQ.NSPIN)IUPDN1=1
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	VHUB(IU,IATOM,IUPDN,    1)=(0.D0,0.D0)
	VHUB(IU,IATOM,IUPDN,NORBS)=(0.D0,0.D0)
	DO L1=0,ILSB(ISORT)                       !!! over l1
	DO M1=-L1,L1                              !!! over m1
	DO K1=-L1,L1                              !!! over k1
	L1M1=L1*(L1+1)+M1+1
	L1K1=L1*(L1+1)+K1+1
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+K1)+L1+M1+1  !  (l1,m1 k1) packed
	IF(ISCHM.EQ.11.OR.ISCHM.EQ.21)THEN
C  Original LDA+U after Liechtenstein et.al (Rotationally Invariant)
C  dVup=U*Ndn+(U-J)Nup-d*[<U>(<N>-1/2)+<J>(<Nup>-1/2)]
C  dVdn=U*Nup+(U-J)Ndn-d*[<U>(<N>-1/2)+<J>(<Ndn>-1/2)]
      VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &UMAT(IU,IU1)*DHUB(IU1,IATOM,IUPDN1,1)+(UMAT(IU,IU1)-
     &GMAT(IU,IU1))*DHUB(IU1,IATOM,IUPDN,1)
	ELSEIF(ISCHM.EQ.12.OR.ISCHM.EQ.22)THEN
C  LDA+U with another double counting.
C  dVup=U*[Ndn-Ndn_LDA]+(U-J)[Nup-Nup_LDA]
C  dVdn=U*[Nup-Nup_LDA]+(U-J)[Ndn-Ndn_LDA]
	VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &(DHUB(IU1,IATOM,IUPDN1,1)-DLDA(IU1,IATOM,IUPDN1,1))*
     & UMAT(IU,IU1)+(DHUB(IU1,IATOM,IUPDN,1)-
     & DLDA(IU1,IATOM,IUPDN,1))*(UMAT(IU,IU1)-GMAT(IU,IU1))
	ELSEIF(ISCHM.EQ.13.OR.ISCHM.EQ.23)THEN
C  LDA+U with average occupancies.
C  dVup=U*[Ndn-Nav]+(U-J)[Nup-Nav]
C  dVdn=U*[Nup-Nav]+(U-J)[Ndn-Nav]
	VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &(DHUB(IU1,IATOM,IUPDN1,1)-DAVR(IU1,IATOM))*UMAT(IU,IU1)+
     &(DHUB(IU1,IATOM,IUPDN,1)-DAVR(IU1,IATOM))*
     &(UMAT(IU,IU1)-GMAT(IU,IU1))
	ELSEIF((ISCHM.EQ.14.OR.ISCHM.EQ.24).
     &            AND.K1.EQ.K.AND.M1.EQ.M)THEN
C  LDA+U in the spherically-averaged form. There is no matrix multiplication
C  V(j,l,sigma)=(Ubar-Jbar)*{0.5*delta(j,l)-rho(j,l,sigma)}
      VHUB(IU,IATOM,IUPDN,1)=-(UBAR(L,L1)-GBAR(L,L1))*
     &DHUB(IU,IATOM,IUPDN,1)
	ELSEIF(ISCHM.EQ.15.OR.ISCHM.EQ.25)THEN
C  Rotationally Invariant LDA+U without U matrix
      VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)-
     &GMAT(IU,IU1)*DHUB(IU1,IATOM,IUPDN,1)
	ENDIF
C ADD RELATIVISTIC CORRECTIONS
C Note that VHUB here is already matrix element <B^{+/-}*S_{+/-}>/sqrt(2)
C not just a field <B^{+/-}, additional 0.5 is due to various sqrt(2) introduced.
	IF(NORBS.EQ.2)
     &VHUB(IU,IATOM,IUPDN,NORBS)=VHUB(IU,IATOM,IUPDN,NORBS)-
     &0.5D0*GMAT(IU,IU1)*DHUB(IU1,IATOM,IUPDN,NORBS)
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
C SUBTRACT DOUBLE COUNTING TERMS
      IF(K.EQ.M.AND.(ISCHM.EQ.11.OR.ISCHM.EQ.21))
     &VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)-
     &UBAR(L,L1)*DBAR(L1,IATOM,IUPDN1,1)-
     &(UBAR(L,L1)-GBAR(L,L1))*DBAR(L1,IATOM,IUPDN,1)+
     &0.5D0*(UBAR(L,L1)-GBAR(L,L1))
      IF(K.EQ.M.AND.(ISCHM.EQ.14.OR.ISCHM.EQ.24))
     &VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &0.5D0*(UBAR(L,L1)-GBAR(L,L1))
      IF(K.EQ.M.AND.(ISCHM.EQ.15.OR.ISCHM.EQ.25))
     &VHUB(IU,IATOM,IUPDN,1)=VHUB(IU,IATOM,IUPDN,1)+
     &GBAR(L,L1)*DBAR(L1,IATOM,IUPDN,1)
	IF(K.EQ.M.AND.NORBS.EQ.2)
     &VHUB(IU,IATOM,IUPDN,NORBS)=VHUB(IU,IATOM,IUPDN,NORBS)+
     &0.5D0*GBAR(L,L1)*DBAR(L1,IATOM,IUPDN,NORBS)
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
1000	CONTINUE                                  !!! over atoms
C MEMORY DEALLOCATIONS
	DEALLOCATE(UMAT,GMAT,UBAR,GBAR,DBAR,DAVR)
	END




	SUBROUTINE HUBRENER(EHUB)
**********************************************************
*  Computes Hubbard correction to the LDA energy in      *
*  the LDA+U scheme.                                     *
*  The important parameter is ISCHM=0,11,12,13 which is  *
*  in MODULE  COM_HUBDAT. See formulae below for a       *
*  particular scheme.                                    *
*  Last revised 29.05.97                                 *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL,ONLY: IDMF                              !!! from readini
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB   !!! from readini
      USE COM_PANPAR ,ONLY: LTPAN,LSPAN                       !!! from readini
      USE COM_HUBDAT ,ONLY: NHUB,MCRL,LCRL,ISCHM              !!! from readhub
      USE COM_HUBCOM ,ONLY: DHUB,VHUB,DLDA,VCNS               !!! from readhub
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: UMAT(:,:)
	REAL*8, ALLOCATABLE :: GMAT(:,:)
	REAL*8, ALLOCATABLE :: UBAR(:,:)
	REAL*8, ALLOCATABLE :: GBAR(:,:)
	COMPLEX*16, ALLOCATABLE :: DBAR(:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DAVR(:,:)
	DATA PI/3.141592653589793D0/
	EHUB=0.D0
	IF(IDMF.NE.1)RETURN
C MEMORY ALLOCATIONS
	ALLOCATE(UMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(GMAT(LIMB2MEM,LIMB2MEM))
	ALLOCATE(UBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(GBAR(0:LMAXBMEM,0:LMAXBMEM))
	ALLOCATE(DBAR(0:LMAXBMEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DAVR(LIMB2MEM,NATOMMEM))
C DO GLOBALLY OVER ATOMS
	ETHUB=0.D0
	EBHUB=0.D0
	ECEXT=0.D0
	DO 1000 IATOM=1,NATOM                     !!! over atoms
	ISORT=IS(IATOM)
      CALL HUBMAT(IATOM,UMAT,GMAT,UBAR,GBAR) ! set Hubbard and Exchange matrices
C SET AVERAGE OCCUPANCIES <Nup> and <Ndn>
	DO IORBS=1,NORBS                          !!! over iorbs
	DO IUPDN=1,NSPIN                          !!! over spins
	DO L=0,ILSB(ISORT)                        !!! over l
	DBAR(L,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
	DO M=-L,L                                 !!! over m
	LM=L*(L+1)+M+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	DBAR(L,IATOM,IUPDN,IORBS)=DBAR(L,IATOM,IUPDN,IORBS)+
     &DHUB(IU,IATOM,IUPDN,IORBS)
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
C SET AVERAGE OVER SPINS ONLY
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      DAVR(IU,IATOM)=(DHUB(IU,IATOM,1,1)+
     &DHUB(IU,IATOM,NSPIN,1))/2.D0
	ENDDO                                     !!! over m
	ENDDO                                     !!! over k
	ENDDO                                     !!! over l
C SET HUBBARD CORRECTION TO THE TOTAL ENERGY
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DO L1=0,ILSB(ISORT)                       !!! over l1
	DO M1=-L1,L1                              !!! over m1
	DO K1=-L1,L1                              !!! over k1
	L1M1=L1*(L1+1)+M1+1
	L1K1=L1*(L1+1)+K1+1
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+K1)+L1+M1+1  !  (l1,m1 k1) packed
	IF(ISCHM.EQ.11.OR.ISCHM.EQ.21)THEN
C Original LDA+U after Liechtenstein et.al (Rotationally Invariant)
      ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*(
     &DHUB(IU ,IATOM,1,1)*DHUB(IU1,IATOM,NSPIN,1)+
     &DHUB(IU1,IATOM,1,1)*DHUB(IU ,IATOM,NSPIN,1))
      ETHUB=ETHUB+0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*(
     &DHUB(IU ,IATOM,1    ,1)*DHUB(IU1,IATOM,1    ,1)+
     &DHUB(IU1,IATOM,NSPIN,1)*DHUB(IU ,IATOM,NSPIN,1))
	ELSEIF(ISCHM.EQ.12.OR.ISCHM.EQ.22)THEN
C LDA+U with another double counting.
	ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*
     &(DHUB(IU ,IATOM,1    ,1)-DLDA(IU ,IATOM,1    ,1))*
     &(DHUB(IU1,IATOM,NSPIN,1)-DLDA(IU1,IATOM,NSPIN,1))
	ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*
     &(DHUB(IU ,IATOM,NSPIN,1)-DLDA(IU ,IATOM,NSPIN,1))*
     &(DHUB(IU1,IATOM,1    ,1)-DLDA(IU1,IATOM,1    ,1))
	ETHUB=ETHUB+0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*
     &(DHUB(IU ,IATOM,1    ,1)-DLDA(IU ,IATOM,1    ,1))*
     &(DHUB(IU1,IATOM,1    ,1)-DLDA(IU1,IATOM,1    ,1))
	ETHUB=ETHUB+0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*
     &(DHUB(IU ,IATOM,NSPIN,1)-DLDA(IU ,IATOM,NSPIN,1))*
     &(DHUB(IU1,IATOM,NSPIN,1)-DLDA(IU1,IATOM,NSPIN,1))
	ELSEIF(ISCHM.EQ.13.OR.ISCHM.EQ.23)THEN
C LDA+U with average occupancies.
	ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*
     &(DHUB(IU ,IATOM,1    ,1)-DAVR(IU ,IATOM))*
     &(DHUB(IU1,IATOM,NSPIN,1)-DAVR(IU1,IATOM))
	ETHUB=ETHUB+0.5D0*UMAT(IU,IU1)*
     &(DHUB(IU ,IATOM,NSPIN,1)-DAVR(IU ,IATOM))*
     &(DHUB(IU1,IATOM,1    ,1)-DAVR(IU1,IATOM))
	ETHUB=ETHUB+0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*
     &(DHUB(IU ,IATOM,1    ,1)-DAVR(IU ,IATOM))*
     &(DHUB(IU1,IATOM,1    ,1)-DAVR(IU1,IATOM))
	ETHUB=ETHUB+0.5D0*(UMAT(IU,IU1)-GMAT(IU,IU1))*
     &(DHUB(IU ,IATOM,NSPIN,1)-DAVR(IU ,IATOM))*
     &(DHUB(IU1,IATOM,NSPIN,1)-DAVR(IU1,IATOM))
	ELSEIF((ISCHM.EQ.14.OR.ISCHM.EQ.24).
     &AND.L1.EQ.L.AND.K1.EQ.K.AND.M1.EQ.M)THEN
C LDA+U in the spherically-averaged form. There is no matrix multiplication
      ETHUB=ETHUB-0.5D0*(UBAR(L,L)-GBAR(L,L))*0.5D0*DBLE(NSPIN)*
     &(DHUB(IU,IATOM,NSPIN,1)*DCONJG(DHUB(IU,IATOM,NSPIN,1))+
     & DHUB(IU,IATOM,1    ,1)*DCONJG(DHUB(IU,IATOM,1    ,1)))
      IF(M.EQ.0.AND.K.EQ.0)
     &ETHUB=ETHUB+0.5D0*(UBAR(L,L)-GBAR(L,L))*
     &(DBAR(L,IATOM,1,1)+DBAR(L,IATOM,NSPIN,1))/2.D0*DBLE(NSPIN)
	ELSEIF(ISCHM.EQ.15.OR.ISCHM.EQ.25)THEN
C LDA+U without U
      ETHUB=ETHUB-0.5D0*GMAT(IU,IU1)*(
     &DHUB(IU ,IATOM,1    ,1)*DHUB(IU1,IATOM,1    ,1)+
     &DHUB(IU1,IATOM,NSPIN,1)*DHUB(IU ,IATOM,NSPIN,1))
	ENDIF
C Relativistic corrections, additional -0.5=-1/sqrt(2)/sqrt(2) appears here
	IF(NORBS.EQ.2) 
     &ETHUB=ETHUB-0.5D0*GMAT(IU,IU1)*(-0.5D0)*(
     &DHUB(IU,IATOM,1,NORBS)*DHUB(IU1,IATOM,NSPIN,NORBS)+
     &DHUB(IU,IATOM,NSPIN,NORBS)*DHUB(IU1,IATOM,1,NORBS))
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
C SUBTRACT DOUBLE COUNTING TERMS
      IF(ISCHM.EQ.11.OR.ISCHM.EQ.21)
     &ETHUB=ETHUB-0.5D0*UBAR(L,L)*(
     &DBAR(L,IATOM,1,1)*DBAR(L,IATOM,NSPIN,1)+
     &DBAR(L,IATOM,1,1)*DBAR(L,IATOM,NSPIN,1))
     &           -0.5D0*(UBAR(L,L)-GBAR(L,L))*(
     &DBAR(L,IATOM,1    ,1)*DBAR(L,IATOM,1    ,1)+
     &DBAR(L,IATOM,NSPIN,1)*DBAR(L,IATOM,NSPIN,1))
     &           +0.5D0*(UBAR(L,L)-GBAR(L,L))*(
     &DBAR(L,IATOM,1    ,1)+DBAR(L,IATOM,NSPIN,1))
      IF(ISCHM.EQ.15.OR.ISCHM.EQ.25)
     &ETHUB=ETHUB+0.5D0*GBAR(L,L)*(
     &DBAR(L,IATOM,1,    1)*DBAR(L,IATOM,1,    1)+
     &DBAR(L,IATOM,NSPIN,1)*DBAR(L,IATOM,NSPIN,1))
	IF(NORBS.EQ.2) 
     &ETHUB=ETHUB+0.5D0*GBAR(L,L)*(-0.5D0)*(
     &DBAR(L,IATOM,1,NORBS)*DBAR(L,IATOM,NSPIN,NORBS)+
     &DBAR(L,IATOM,NSPIN,NORBS)*DBAR(L,IATOM,1,NORBS))
	ENDDO                                     !!! over l
C SET HUBBARD CORRECTION TO THE BAND ENERGY
	DO IORBS=1,NORBS                          !!! over spins
	DO IUPDN=1,NSPIN                          !!! over spins
	IF(IORBS.EQ.1)THEN
	IUPDN1=IUPDN
	SIGNF=+1.D0
	ELSEIF(IORBS.EQ.2)THEN
	IUPDN1=NSPIN-IUPDN+1
	SIGNF=-1.D0
	ENDIF
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	EBHUB=EBHUB+SIGNF*DHUB(IU,IATOM,IUPDN,IORBS)*
     &VHUB(IU,IATOM,IUPDN1,IORBS)
	ECEXT=ECEXT+SIGNF*DHUB(IU,IATOM,IUPDN,IORBS)*
     &VCNS(IU,IATOM,IUPDN1,IORBS)
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
	ENDDO                                     !!! over spins
	ENDDO                                     !!! over iorbs
1000	CONTINUE                                  !!! over atoms
	IF(ISCHM.GE.10.AND.ISCHM.LE.19)THEN
	EHUB=ETHUB-EBHUB
	WRITE(IUN,1010)ETHUB,EBHUB
	ELSEIF(ISCHM.EQ.20)THEN
	EHUB=-ECEXT
	WRITE(IUN,1020)ECEXT
	ELSEIF(ISCHM.GE.21.AND.ISCHM.LE.29)THEN
	EHUB=ETHUB-EBHUB-ECEXT
	WRITE(IUN,1010)ETHUB,EBHUB
	WRITE(IUN,1020)ECEXT
	ENDIF
C MEMORY DEALLOCATIONS
	DEALLOCATE(UMAT)
	DEALLOCATE(GMAT)
	DEALLOCATE(UBAR)
	DEALLOCATE(GBAR)
	DEALLOCATE(DBAR)
	DEALLOCATE(DAVR)
1010  FORMAT(1X,'E{hub} : Et,bnd-parts are',2(1X,G14.7))
1020  FORMAT(1X,'E{ext} : due to constrain',1X,G14.7)
	END



	SUBROUTINE HUBMAT(IATOM,UMAT,GMAT,UBAR,GBAR)
**********************************************************
*  Computes Hubbard and Exchange matrices for a          *
*  given atom. Knowledge of Slater itegrals is required. *
*                                                        *
*  Notations:                                            *
*  If <ij|1/r|kl> stands for a standard definition,      *
*     <ij|1/r|ij> is a Hubbard-type interaction and      *
*     <ij|1/r|ji> is an exchange interaction, then:      *
*                                                        *
*1)Hubbard matrix in the lm-representation is given by   *
*     <lm,l'm'|1/r|lk,l'k'>=U(lmk,l'm'k')                *
*  and both LMK and L'M'K' can be packed into one index. *
*  Therefore UMAT(IU,IU1)=U(lmk,l'm'k')                  *
*                                                        *
*2)Exchange matrix in the lm-representation             *
*     <lm,l'm'|1/r|l'k',lk>=J(lmk,l'm'k')                *
*  and both LMK and L'M'K' can be packed into one index. *
*  Therefore GMAT(IU,IU1)=J(lmk,l'm'k')                  *
*                                                        *
*  Definitions for Slater integrals:                     *
*  For Hubbard-type interaction it is useful to          *
*  consider      /   l"  l"+1   2     2                  *
*     Fu(l",ll')=\ {r  /r'   } P (r)*P (r')              *
*                /              l     l'                 *
*  For Exchange interactions it is useful to define      *
*                /   l"  l"+1                            *
*     Fj(l",ll')=\ {r  /r'   } P (r)*P (r)*P (r')*P (r') *
*                /              l     l'    l     l'     *
*  Fu(l",ll')=FSLU(L",LL') and                           *
*  Fj(l",ll')=FSLG(L",LL') are input matrices to the     *
*  program.                                              *
*                                                        *
*  Last revised 21.11.96                                 *
**********************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM  ,ONLY: NATOM,NSORT,NSPIN,NORBS,IS,ILSB   !!! from readini
      USE COM_PANPAR ,ONLY: LSPAN,LTPAN                       !!! from readini
      USE COM_FSLATR ,ONLY: FSLG,FSLU                         !!! from readhub
      USE COM_GAUNT  ,ONLY: C                                 !!! from gdata
      USE COM_UNIT   ,ONLY: IUN,IUR,IUT                       !!! from setup
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION UMAT(LIMB2MEM,LIMB2MEM),GMAT(LIMB2MEM,LIMB2MEM) 
     &   ,UBAR(0:LMAXBMEM,0:LMAXBMEM),GBAR(0:LMAXBMEM,0:LMAXBMEM)
	DATA PI/3.141592653589793D0/
C START CODE
	ISORT=IS(IATOM)
	DO L=0,ILSB(ISORT)                        !!! over l
	DO M=-L,L                                 !!! over m
	DO K=-L,L                                 !!! over k
	LM=L*(L+1)+M+1
	LK=L*(L+1)+K+1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DO L1=0,ILSB(ISORT)                       !!! over l1
	DO M1=-L1,L1                              !!! over m1
	DO K1=-L1,L1                              !!! over k1
	L1M1=L1*(L1+1)+M1+1
	L1K1=L1*(L1+1)+K1+1
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+K1)+L1+M1+1  !  (l1,m1 k1) packed
C BUILT HUBBARD  MATRIX "UMAT" FROM THE SLATER INTEGRALS "FSLU"
	UMAT(IU,IU1)=(0.D0,0.D0)
C U(lmk; l'm'k') corresponds to <lm l'm'|1/r|lk l'k'>
	DO L2=0,MIN(2*L,2*L1),2                   !!! over l2
	M2=M-K
	K2=K1-M1
	IF(M2.EQ.K2.AND.ABS(M2).LE.L2)THEN        !!! selection rule
	UMAT(IU,IU1)=UMAT(IU,IU1)+4.D0*PI/DBLE(2*L2+1)*
     &  FSLU(L2,L,L1,IATOM)*C(L2/2,LM,LK)*C(L2/2,L1K1,L1M1)
	ENDIF                                     !!! selection rule
	ENDDO                                     !!! over l2
C BUILT EXCHANGE MATRIX "GMAT" FROM THE SLATER INTEGRALS "FSLG"
	GMAT(IU,IU1)=(0.D0,0.D0)
C U(lmk; l'm'k') corresponds to <lm l'm'|1/r|lk l'k'>
	DO L2=ABS(L-L1),L+L1,2                    !!! over l2
	M2=M-K1
	K2=K-M1
	IF(M2.EQ.K2.AND.ABS(M2).LE.L2)THEN        !!! selection rule
	GMAT(IU,IU1)=GMAT(IU,IU1)+4.D0*PI/DBLE(2*L2+1)*
     &FSLG(L2,L,L1,IATOM)*C(L2/2,LM,L1K1)*C(L2/2,LK,L1M1)
	ENDIF                                     !!! selection rule
	ENDDO                                     !!! over l2
	ENDDO                                     !!! over k1
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over k
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l
C SET AVERAGE VALUES <U> AND <J>
	DO L=0,ILSB(ISORT)                        !!! over l
	DO L1=0,ILSB(ISORT)                       !!! over l1
	UBAR(L,L1)=0.D0
	GBAR(L,L1)=0.D0
	DLL1=1.D0/DBLE(2*L+1)/DBLE(2*L1+1)
	DLL=0.D0
	IF(L.NE.0)
     &DLL =1.D0/DBLE(2*L  )/DBLE(2*L1+1) ! not sure, must be checked !
	DO M=-L,L                                 !!! over m
	DO M1=-L1,L1                              !!! over m1
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M+1           !  (l,m m) packed
	IU1=L1*(2*L1-1)*(2*L1+1)/3+(2*L1+1)*(L1+M1)+L1+M1+1  !  (l1,m1 m1) packed
	UBAR(L,L1)=UBAR(L,L1)+DLL1*UMAT(IU,IU1)
	GBAR(L,L1)=GBAR(L,L1)+DLL1*UMAT(IU,IU1)-
     &DLL*(UMAT(IU,IU1)-GMAT(IU,IU1))
	ENDDO                                     !!! over m1
	ENDDO                                     !!! over m
	ENDDO                                     !!! over l1
	ENDDO                                     !!! over l
	END

